# ------------------------------------------------------------------------------
#  dragsite.tcl
#  This file is part of Unifix BWidget Toolkit
# ------------------------------------------------------------------------------
#  Index of commands:
#     - DragSite::include
#     - DragSite::setdrag
#     - DragSite::register
#     - DragSite::_init_drag
#     - DragSite::_end_drag
#     - DragSite::_update_operation
# ------------------------------------------------------------------------------

namespace eval DragSite {
    Widget::declare DragSite {
        {-dragevent     String "" 0}
        {-draginitcmd   String "" 0}
        {-dragendcmd    String "" 0}
    }

    variable _topw ".drag"
    variable _opw  ".drag.\#op"
    variable _img
    variable _tabops

    array set _img \
        [list \
             move @[file join $env(BWIDGET_LIBRARY) "images" "opmove.xbm"] \
             copy @[file join $env(BWIDGET_LIBRARY) "images" "opcopy.xbm"] \
             link @[file join $env(BWIDGET_LIBRARY) "images" "oplink.xbm"]]

    proc use {} {}

    bind DragTop <KeyPress-Shift_L>     {DragSite::_update_operation [expr %s | 1]}
    bind DragTop <KeyPress-Shift_R>     {DragSite::_update_operation [expr %s | 1]}
    bind DragTop <KeyPress-Control_L>   {DragSite::_update_operation [expr %s | 4]}
    bind DragTop <KeyPress-Control_R>   {DragSite::_update_operation [expr %s | 4]}
    bind DragTop <KeyPress-Alt_L>       {DragSite::_update_operation [expr %s | 88]}
    bind DragTop <KeyPress-Alt_R>       {DragSite::_update_operation [expr %s | 88]}
    bind DragTop <KeyPress-Meta_L>      {DragSite::_update_operation [expr %s | 88]}
    bind DragTop <KeyPress-Meta_R>      {DragSite::_update_operation [expr %s | 88]}

    bind DragTop <KeyRelease-Shift_L>   {DragSite::_update_operation [expr %s & ~1]}
    bind DragTop <KeyRelease-Shift_R>   {DragSite::_update_operation [expr %s & ~1]}
    bind DragTop <KeyRelease-Control_L> {DragSite::_update_operation [expr %s & ~4]}
    bind DragTop <KeyRelease-Control_R> {DragSite::_update_operation [expr %s & ~4]}
    bind DragTop <KeyRelease-Alt_L>     {DragSite::_update_operation [expr %s & ~88]}
    bind DragTop <KeyRelease-Alt_R>     {DragSite::_update_operation [expr %s & ~88]}
    bind DragTop <KeyRelease-Meta_L>    {DragSite::_update_operation [expr %s & ~88]}
    bind DragTop <KeyRelease-Meta_L>    {DragSite::_update_operation [expr %s & ~88]}
}


# ------------------------------------------------------------------------------
#  Command DragSite::include
# ------------------------------------------------------------------------------
proc DragSite::include { class type event } {
    set dragoptions {
        {-dragenabled Boolean 0  0}
        {-draginitcmd String  "" 0}
        {-dragendcmd  String  "" 0}
    }
    lappend dragoptions \
        [list -dragtype  String $type  0] \
        [list -dragevent String $event 0]
    Widget::declare $class $dragoptions
}


# ------------------------------------------------------------------------------
#  Command DragSite::setdrag
#  Widget interface to register
# ------------------------------------------------------------------------------
proc DragSite::setdrag { path subpath initcmd endcmd {force 0}} {
    set cen       [Widget::hasChanged $path -dragenabled en]
    set cdragevt  [Widget::hasChanged $path -dragevent   dragevt]
    if { $en } {
        if { $force || $cen || $cdragevt } {
            register $subpath \
                -draginitcmd $initcmd \
                -dragendcmd  $endcmd  \
                -dragevent   $dragevt
        }
    } else {
        register $subpath
    }
}


# ------------------------------------------------------------------------------
#  Command DragSite::register
# ------------------------------------------------------------------------------
proc DragSite::register { path args } {
    upvar \#0 DragSite::$path drag

    if { [info exists drag] } {
        bind $path $drag(evt) {}
        unset drag
    }
    Widget::init DragSite .drag$path $args
    set event   [Widget::cget .drag$path -dragevent]
    set initcmd [Widget::cget .drag$path -draginitcmd]
    set endcmd  [Widget::cget .drag$path -dragendcmd]
    Widget::destroy .drag$path
    if { $initcmd != "" && $event != "" } {
        if { [catch {bind $path $event "DragSite::_init_drag $path %s %X %Y"} errmsg] } {
            return -code error $errmsg
        }
        set drag(initcmd) $initcmd
        set drag(endcmd)  $endcmd
        set drag(evt)     $event
    }
}


# ------------------------------------------------------------------------------
#  Command DragSite::_init_drag
# ------------------------------------------------------------------------------
proc DragSite::_init_drag { source state X Y } {
    variable _topw
    variable _opw
    variable _tabops
    upvar \#0 DragSite::$source drag

    destroy  $_topw
    toplevel $_topw
    wm withdraw $_topw
    wm overrideredirect $_topw 1

    set info [uplevel \#0 $drag(initcmd) [list $source $X $Y .drag]]
    if { $info != "" } {
        set type [lindex $info 0]
        set ops  [lindex $info 1]
        set data [lindex $info 2]

        catch {unset _tabops}
        array set _tabops $ops
        set _tabops(force) copy

        if { [winfo children $_topw] == "" } {
            if { $type == "BITMAP" || $type == "IMAGE" } {
                label $_topw.l -image [Bitmap::get dragicon] -relief flat -bd 0
            } else {
                label $_topw.l -image [Bitmap::get dragfile] -relief flat -bd 0
            }
            pack  $_topw.l
        }
        label $_opw -relief flat -bd 0 -highlightthickness 0 \
            -foreground black -background white
        _update_operation $state
        wm geometry $_topw +[expr $X+1]+[expr $Y+1]
        wm deiconify $_topw
        tkwait visibility $_topw
        BWidget::grab  set $_topw
        BWidget::focus set $_topw

        bindtags $_topw [list $_topw DragTop]
        DropSite::_init_drag $_topw $source $state $X $Y $type $data
    } else {
        destroy $_topw
    }
}


# ------------------------------------------------------------------------------
#  Command DragSite::_end_drag
# ------------------------------------------------------------------------------
proc DragSite::_end_drag { source target op type data result } {
    variable _topw
    upvar \#0 DragSite::$source drag

    BWidget::grab  release $_topw
    BWidget::focus release $_topw
    destroy $_topw
    if { $drag(endcmd) != "" } {
        uplevel \#0 $drag(endcmd) [list $source $target $op $type $data $result]
    }
}


# ------------------------------------------------------------------------------
#  Command DragSite::_update_operation
# ------------------------------------------------------------------------------
proc DragSite::_update_operation { state } {
    variable _img
    variable _opw
    variable _tabops

    if { $state & 88 } {
        set mod "alt"
    } elseif { $state & 4 } {
        set mod "control"
    } elseif { $state & 1 } {
        set mod "shift"
    } else {
        set mod "default"
    }
    if { ![info exists _tabops($mod)] } {
        if { [info exists _tabops(default)] } {
             set mod "default"
         } else {
             set mod "force"
         }
    }
    if { [info exists DropSite::_listops] &&
         ![info exists DropSite::_listops($_tabops($mod))] } {
        set mod "force"
    }
    if { $mod == "default" } {
        catch {place forget $_opw}
    } else {
        catch {
            $_opw configure -bitmap $_img($_tabops($mod))
            place $_opw -relx 1 -rely 1 -anchor se
        }
    }
    return $_tabops($mod)
}

