# ------------------------------------------------------------------------------
#  dropsite.tcl
#  This file is part of Unifix BWidget Toolkit
# ------------------------------------------------------------------------------
#  Index of commands:
#     - DropSite::include
#     - DropSite::setdrop
#     - DropSite::register
#     - DropSite::setcursor
#     - DropSite::_init_drag
#     - DropSite::_motion
#     - DropSite::_release
# ------------------------------------------------------------------------------


namespace eval DropSite {
    Widget::declare DropSite {
        {-dropovercmd String "" 0}
        {-dropcmd     String "" 0}
        {-droptypes   String "" 0}
    }

    proc use { } {}

    variable _topw
    variable _current ""
    variable _statut  0
    variable _listops
}


# ------------------------------------------------------------------------------
#  Command DropSite::include
# ------------------------------------------------------------------------------
proc DropSite::include { class types } {
    set dropoptions {
        {-dropenabled Boolean 0  0}
        {-dropovercmd String  "" 0}
        {-dropcmd     String  "" 0}
    }
    lappend dropoptions [list -droptypes String $types 0]
    Widget::declare $class $dropoptions
}


# ------------------------------------------------------------------------------
#  Command DropSite::setdrop
#  Widget interface to register
# ------------------------------------------------------------------------------
proc DropSite::setdrop { path subpath dropover drop {force 0}} {
    set cen    [Widget::hasChanged $path -dropenabled en]
    set ctypes [Widget::hasChanged $path -droptypes   types]
    if { $en } {
        if { $force || $cen || $ctypes } {
            register $subpath \
                -droptypes   $types \
                -dropcmd     $drop  \
                -dropovercmd $dropover
        }
    } else {
        register $subpath
    }
}


# ------------------------------------------------------------------------------
#  Command DropSite::register
# ------------------------------------------------------------------------------
proc DropSite::register { path args } {
    upvar \#0 DropSite::$path drop

    Widget::init DropSite .drop$path $args
    if { [info exists drop] } {
        unset drop
    }
    set dropcmd [Widget::cget .drop$path -dropcmd]
    set types   [Widget::cget .drop$path -droptypes]
    set overcmd [Widget::cget .drop$path -dropovercmd]
    Widget::destroy .drop$path
    if { $dropcmd != "" && $types != "" } {
        set drop(dropcmd) $dropcmd
        set drop(overcmd) $overcmd
        set drop(types)   $types
        foreach {type ops} $types {
            set drop($type,ops) {}
            foreach op $ops {
                set drop($type,$op) 1
                lappend drop($type,ops) $op 1
            }
            if { ![info exist drop($type,copy)] } {
                set drop($type,copy) 1
                lappend drop($type,ops) copy 1
            }
        }
    }
}


# ------------------------------------------------------------------------------
#  Command DropSite::setcursor
# ------------------------------------------------------------------------------
proc DropSite::setcursor { cursor } {
    catch {.drag configure -cursor $cursor}
}


# ------------------------------------------------------------------------------
#  Command DropSite::_init_drag
# ------------------------------------------------------------------------------
proc DropSite::_init_drag { top source state X Y type data } {
    variable _current
    variable _statut
    variable _listops

    set _current ""
    set _statut  0
    catch {unset _listops}

    bind $top <ButtonRelease> "DropSite::_release $top $source %s %X %Y [list $type $data]"
    bind $top <Motion>        "DropSite::_motion  $top $source %s %X %Y [list $type $data]"

    _motion $top $source $state $X $Y $type $data
}


# ------------------------------------------------------------------------------
#  Command DropSite::_motion
# ------------------------------------------------------------------------------
proc DropSite::_motion { top source state X Y type data } {
    variable _current
    variable _statut
    variable _listops

    set script [bind $top <Motion>]
    bind $top <Motion> {}
    wm geometry $top "+[expr {$X+1}]+[expr {$Y+1}]"
    update
    if { ![winfo exists $top] } { return }
    set path [winfo containing $X $Y]
    if { $path != $_current } {
        if { $_statut & 1 } {
            upvar \#0 DropSite::$_current drop
            uplevel \#0 $drop(overcmd) [list $_current $source leave $X $Y "" $type $data]
        }
        catch {unset _listops}
        set _current $path
        if { $path != "" } {
            upvar \#0 DropSite::$path drop
            
            if { [info exists drop($type,ops)] } {
                array set _listops $drop($type,ops)
            }
            set op [DragSite::_update_operation $state]
            if { [info exists drop($type,$op)] } {
                if { $drop(overcmd) != "" } {
                    set arg     [list $path $source enter $X $Y $op $type $data]
                    set _statut [uplevel \#0 $drop(overcmd) $arg]
                } else {
                    set _statut 2
                    catch {$top configure -cursor based_arrow_down}
                }
                catch {bind $top <Motion> $script}
                return
            }
        } else {
            set op [DragSite::_update_operation $state]
        }
        set _statut 0
        catch {$top configure -cursor dot}
    } elseif { $_statut & 1 } {
        upvar \#0 DropSite::$_current drop
        set op      [DragSite::_update_operation $state]
        set arg     [list $_current $source motion $X $Y $op $type $data]
        set _statut [uplevel \#0 $drop(overcmd) $arg]
    }
    update
    catch {bind $top <Motion> $script}
}



# ------------------------------------------------------------------------------
#  Command DropSite::_release
# ------------------------------------------------------------------------------
proc DropSite::_release { top source state X Y type data } {
    variable _current
    variable _statut

    if { $_statut & 2 } {
        upvar \#0 DropSite::$_current drop

        set op  [DragSite::_update_operation $state]
        set res [uplevel \#0 $drop(dropcmd) [list $_current $source $X $Y $op $type $data]]
        DragSite::_end_drag $source $_current $op $type $data $res
    } else {
        if { $_statut & 1 } {
            # notify leave event
            upvar \#0 DropSite::$_current drop
            uplevel \#0 $drop(overcmd) [list $_current $source leave $X $Y "" $type $data]
        }
        DragSite::_end_drag $source "" "" $type $data 0
    }
}
