# ------------------------------------------------------------------------------
#  labelentry.tcl
#  This file is part of Unifix BWidget Toolkit
# ------------------------------------------------------------------------------
#  Index of commands:
#     - LabelEntry::create
#     - LabelEntry::configure
#     - LabelEntry::cget
# ------------------------------------------------------------------------------

namespace eval LabelEntry {
    Entry::use
    LabelFrame::use

    Widget::bwinclude LabelEntry LabelFrame .labf \
        remove {-relief -borderwidth -focus} \
        rename {-text -label} \
        prefix {label -justify -width -anchor -height -font} \

    Widget::bwinclude LabelEntry Entry .e \
        remove {-fg -bg} \
        rename {-foreground -entryfg -background -entrybg}

    Widget::addmap LabelEntry "" :cmd {-background {}}

    proc ::LabelEntry { path args } { return [eval LabelEntry::create $path $args] }
    proc use { } {}
}


# ------------------------------------------------------------------------------
#  Command LabelEntry::create
# ------------------------------------------------------------------------------
proc LabelEntry::create { path args } {
    Widget::init LabelEntry $path $args

    eval frame $path [Widget::subcget $path :cmd] \
        -relief flat -bd 0 -highlightthickness 0 -takefocus 0 \

    set labf  [eval LabelFrame::create $path.labf [Widget::subcget $path .labf] \
                   -relief flat -borderwidth 0 -focus $path.e]
    set subf  [LabelFrame::getframe $labf]
    set entry [eval Entry::create $path.e [Widget::subcget $path .e]]

    pack $entry -in $subf -fill both -expand yes
    pack $labf  -fill both -expand yes

    bind $path <Destroy> {Widget::destroy %W; rename %W {}}
    bind $path <FocusIn> {focus %W.e}

    rename $path ::$path:cmd
    proc ::$path { cmd args } "return \[eval LabelEntry::\$cmd $path \$args\]"

    return $path
}


# ------------------------------------------------------------------------------
#  Command LabelEntry::configure
# ------------------------------------------------------------------------------
proc LabelEntry::configure { path args } {
    Widget::setoption $path -value [Entry::cget $path.e -text]
    return [Widget::configure $path $args]
}


# ------------------------------------------------------------------------------
#  Command LabelEntry::cget
# ------------------------------------------------------------------------------
proc LabelEntry::cget { path option } {
    Widget::setoption $path -value [Entry::cget $path.e -text]
    return [Widget::cget $path $option]
}
