# ------------------------------------------------------------------------------
#  progressbar.tcl
#  This file is part of Unifix BWidget Toolkit
# ------------------------------------------------------------------------------
#  Index of commands:
#     - ProgressBar::create
#     - ProgressBar::configure
#     - ProgressBar::cget
#     - ProgressBar::_destroy
#     - ProgressBar::_modify
# ------------------------------------------------------------------------------

namespace eval ProgressBar {
    Widget::declare ProgressBar {
        {-background  TkResource ""         0 frame}
        {-foreground  TkResource blue       0 label}
        {-borderwidth TkResource 2          0 frame}
        {-troughcolor TkResource ""         0 {entry -background}}
        {-relief      TkResource sunken     0 label}
        {-orient      Enum       horizontal 1 {horizontal vertical}}
        {-variable    String     ""         0}
        {-width       TkResource 100        0 frame}
        {-height      TkResource 4m         0 frame}
        {-bg          Synonym    -background}
        {-fg          Synonym    -foreground}
        {-bd          Synonym    -borderwidth}
    }

    Widget::addmap ProgressBar "" :cmd {-background {} -width {} -height {}}
    Widget::addmap ProgressBar "" .bar {-troughcolor -background -borderwidth {} -relief {}}

    variable _widget

    proc ::ProgressBar { path args } { return [eval ProgressBar::create $path $args] }
    proc use {} {}
}


# ------------------------------------------------------------------------------
#  Command ProgressBar::create
# ------------------------------------------------------------------------------
proc ProgressBar::create { path args } {
    variable _widget

    Widget::init ProgressBar $path $args

    eval frame $path [Widget::subcget $path :cmd]
    set c  [eval canvas $path.bar [Widget::subcget $path .bar] -highlightthickness 0]
    set fg [Widget::cget $path -foreground]
    if { [Widget::cget $path -orient] == "horizontal" } {
        $path.bar create rectangle -1 0 0 0 -fill $fg -outline $fg -tags rect
    } else {
        $path.bar create rectangle 0 1 0 0 -fill $fg -outline $fg -tags rect
    }

    if { [set _widget($path,var) [Widget::cget $path -variable]] != "" } {
        uplevel \#0 trace variable $_widget($path,var) w [list "ProgressBar::_modify $path"]
        after idle ProgressBar::_modify $path
    }

    bind $path.bar <Destroy>   "ProgressBar::_destroy $path"
    bind $path.bar <Configure> "ProgressBar::_modify $path"

    rename $path ::$path:cmd
    proc ::$path { cmd args } "return \[eval ProgressBar::\$cmd $path \$args\]"

    return $path
}


# ------------------------------------------------------------------------------
#  Command ProgressBar::configure
# ------------------------------------------------------------------------------
proc ProgressBar::configure { path args } {
    variable _widget

    set res [Widget::configure $path $args]

    if { [Widget::hasChanged $path -variable newv] } {
        if { $_widget($path,var) != "" } {
            uplevel \#0 trace vdelete $_widget($path,var) w [list "ProgressBar::_modify $path"]
        }
        if { $newv != "" } {
            set _widget($path,var) $newv
            uplevel \#0 trace variable $newv w [list "ProgressBar::_modify $path"]
            after idle ProgressBar::_modify $path
        } else {
            set _widget($path,var) ""
        }
    }

    if { [Widget::hasChanged $path -borderwidth v] ||
         [Widget::hasChanged $path -orient v] } {
        after idle ProgressBar::_modify $path
    }
    if { [Widget::hasChanged $path -foreground fg] } {
        $path.bar itemconfigure -fill $fg -outline $fg
    }
    return $res
}


# ------------------------------------------------------------------------------
#  Command ProgressBar::cget
# ------------------------------------------------------------------------------
proc ProgressBar::cget { path option } {
    return [Widget::cget $path $option]
}


# ------------------------------------------------------------------------------
#  Command ProgressBar::_destroy
# ------------------------------------------------------------------------------
proc ProgressBar::_destroy { path } {
    variable _widget

    if { $_widget($path,var) != "" } {
        uplevel \#0 trace vdelete $_widget($path,var) w [list "ProgressBar::_modify $path"]
    }
    unset _widget($path,var)
    Widget::destroy $path
    rename $path {}
}


# ------------------------------------------------------------------------------
#  Command ProgressBar::_modify
# ------------------------------------------------------------------------------
proc ProgressBar::_modify { path args } {
    variable _widget

    if { ![GlobalVar::exists $_widget($path,var)] ||
         [set val [GlobalVar::get $_widget($path,var)]] < 0.0 } {
        place forget $path.bar
    } else {
        if { $val > 100.0 } {set val 100.0}
        place $path.bar -relx 0 -rely 0 -relwidth 1 -relheight 1
        set bd [expr {2*[$path.bar cget -bd]}]
        set w  [winfo width  $path.bar]
        set h  [winfo height $path.bar]
        if { [Widget::cget $path -orient] == "horizontal" } {
            $path.bar coords rect -1 0 [expr {$val*$w/100.0}] $h
        } else {
            $path.bar coords rect 0 [expr {$h+1}] $w [expr {$h*(100.0-$val)}]
        }
    }
    update
}
